@@ -13,7 +13,7 @@ fn filters_minsum(b: &mut Bencher) {
     let input = test::black_box(PathBuf::from("tests/files/rgb_8_should_be_rgb_8.png"));
     let png = PngData::new(&input, &Options::default()).unwrap();
 
-    b.iter(|| png.raw.filter_image(RowFilter::MinSum, false));
+    b.iter(|| png.raw.filter_image(FilterStrategy::MinSum, false));
 }
 
 #[bench]
@@ -21,7 +21,7 @@ fn filters_entropy(b: &mut Bencher) {
     let input = test::black_box(PathBuf::from("tests/files/rgb_8_should_be_rgb_8.png"));
     let png = PngData::new(&input, &Options::default()).unwrap();
 
-    b.iter(|| png.raw.filter_image(RowFilter::Entropy, false));
+    b.iter(|| png.raw.filter_image(FilterStrategy::Entropy, false));
 }
 
 #[bench]
@@ -29,7 +29,7 @@ fn filters_bigrams(b: &mut Bencher) {
     let input = test::black_box(PathBuf::from("tests/files/rgb_8_should_be_rgb_8.png"));
     let png = PngData::new(&input, &Options::default()).unwrap();
 
-    b.iter(|| png.raw.filter_image(RowFilter::Bigrams, false));
+    b.iter(|| png.raw.filter_image(FilterStrategy::Bigrams, false));
 }
 
 #[bench]
@@ -37,7 +37,7 @@ fn filters_bigent(b: &mut Bencher) {
     let input = test::black_box(PathBuf::from("tests/files/rgb_8_should_be_rgb_8.png"));
     let png = PngData::new(&input, &Options::default()).unwrap();
 
-    b.iter(|| png.raw.filter_image(RowFilter::BigEnt, false));
+    b.iter(|| png.raw.filter_image(FilterStrategy::BigEnt, false));
 }
 
 #[bench]
@@ -45,5 +45,13 @@ fn filters_brute(b: &mut Bencher) {
     let input = test::black_box(PathBuf::from("tests/files/rgb_8_should_be_rgb_8.png"));
     let png = PngData::new(&input, &Options::default()).unwrap();
 
-    b.iter(|| png.raw.filter_image(RowFilter::Brute, false));
+    b.iter(|| {
+        png.raw.filter_image(
+            FilterStrategy::Brute {
+                num_lines: 4,
+                level: 1,
+            },
+            false,
+        )
+    });
 }
