@@ -212,7 +212,7 @@ losslessly.")
         )
         .arg(
             Arg::new("verbose")
-                .help("Run in verbose mode (use twice to increase verbosity)")
+                .help("Show per-file info (use multiple times for more detail)")
                 .short('v')
                 .long("verbose")
                 .action(ArgAction::Count)
@@ -220,12 +220,20 @@ losslessly.")
         )
         .arg(
             Arg::new("quiet")
-                .help("Run in quiet mode")
+                .help("Suppress all output messages")
                 .short('q')
                 .long("quiet")
                 .action(ArgAction::SetTrue)
                 .conflicts_with("verbose"),
         )
+        .arg(
+            Arg::new("json")
+                .help("Print results as JSON")
+                .short('j')
+                .long("json")
+                .action(ArgAction::SetTrue)
+                .conflicts_with("stdout"),
+        )
         .arg(
             Arg::new("filters")
                 .help("Filters to try (0-9; see '--help' for details)")
