@@ -1,10 +1,12 @@
 use std::{
     fs::remove_file,
     path::{Path, PathBuf},
+    process::Command,
 };
 
 use indexmap::indexset;
 use oxipng::{internal_tests::*, *};
+use serde_json::Value;
 
 const GRAY: u8 = 0;
 const RGB: u8 = 2;
@@ -430,12 +432,9 @@ fn interlaced_0_to_1_other_filter_mode() {
 fn preserve_attrs() {
     let input = PathBuf::from("tests/files/preserve_attrs.png");
 
-    #[cfg(feature = "filetime")]
     let meta_input = input
         .metadata()
-        .expect("unable to get file metadata for output file");
-    #[cfg(feature = "filetime")]
-    let mtime_canon = filetime::FileTime::from_last_modification_time(&meta_input);
+        .expect("unable to get metadata for input file");
 
     let (mut output, opts) = get_opts(&input);
     if let OutFile::Path { preserve_attrs, .. } = &mut output {
@@ -449,17 +448,6 @@ fn preserve_attrs() {
     let output = output.path().unwrap();
     assert!(output.exists());
 
-    #[cfg(feature = "filetime")]
-    let meta_output = output
-        .metadata()
-        .expect("unable to get file metadata for output file");
-    #[cfg(feature = "filetime")]
-    assert_eq!(
-        &mtime_canon,
-        &filetime::FileTime::from_last_modification_time(&meta_output),
-        "expected modification time to be identical to that of input",
-    );
-
     match PngData::new(output, &opts) {
         Ok(x) => x,
         Err(x) => {
@@ -468,9 +456,21 @@ fn preserve_attrs() {
         }
     };
 
-    remove_file(output).ok();
+    let meta_output = output
+        .metadata()
+        .expect("unable to get metadata for output file");
+    assert_eq!(
+        meta_input.modified().ok(),
+        meta_output.modified().ok(),
+        "expected modification time to be identical to that of input",
+    );
+    assert_eq!(
+        meta_input.permissions(),
+        meta_output.permissions(),
+        "expected permissions to be identical to that of input",
+    );
 
-    // TODO: Actually check permissions
+    remove_file(output).ok();
 }
 
 #[test]
@@ -664,3 +664,39 @@ fn zopfli_mode() {
         BitDepth::Eight,
     );
 }
+#[test]
+fn json_success() {
+    let path = "tests/files/json.png";
+    let output = Command::new(env!("CARGO_BIN_EXE_oxipng"))
+        .args(["--json", "--nx", "--nz", path])
+        .output()
+        .expect("Failed to run executable");
+    assert!(output.status.success());
+    let stdout = String::from_utf8_lossy(&output.stdout);
+    let data: Value = serde_json::from_str(&stdout).expect("Failed to parse JSON");
+    let result = &data["results"][0];
+    assert_eq!(result["input"], path);
+    assert_eq!(result["status"], "success");
+    assert_eq!(result["output"], path);
+    assert_eq!(result["insize"], result["outsize"]);
+}
+
+#[test]
+fn json_dry_run() {
+    let path = "tests/files/json.png";
+    let path2 = "tests/files/escape chars \\ \" \n \t \r \x08 \x0c.png";
+    let output = Command::new(env!("CARGO_BIN_EXE_oxipng"))
+        .args(["--json", "--dry-run", path, path2])
+        .output()
+        .expect("Failed to run executable");
+    assert!(output.status.success());
+    let stdout = String::from_utf8_lossy(&output.stdout);
+    let data: Value = serde_json::from_str(&stdout).expect("Failed to parse JSON");
+    let result = &data["results"][0];
+    assert_eq!(result["input"], path);
+    assert_eq!(result["status"], "success");
+    assert_eq!(result["output"], Value::Null);
+    let result2 = &data["results"][1];
+    assert_eq!(result2["input"], path2);
+    assert_eq!(result2["status"], "error");
+}
